###################################################################################################
# _blobfields.py
#
# Implementation of classes MyFile, MyImage, etc. (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from OFS.Image import Image, File
from cStringIO import StringIO
import copy
import sys
# Product Imports.
import _fileutil
import _globals
import _mimetypes


#----------------------------------------------------------------------------------------------
# _blobfields.hex2bin:
#----------------------------------------------------------------------------------------------
def hex2bin(m):
  return ''.join(map(lambda x: chr(16*int('0x%s'%m[x*2],0)+int('0x%s'%m[x*2+1],0)),range(0,len(m)/2)))

#----------------------------------------------------------------------------------------------
# _blobfields.hex2bin:
#----------------------------------------------------------------------------------------------
def bin2hex(m):
  return ''.join(map(lambda x: hex(ord(x)/16)[-1]+hex(ord(x)%16)[-1],m))


#----------------------------------------------------------------------------------------------
# _blobfields.recurse_downloadRessources:
#
# Download from ZODB to file-system during Export.
#----------------------------------------------------------------------------------------------
def recurse_downloadRessources(self, folder='.'):
  #++ _globals.writeLog("[%s.recurse_downloadRessources]: %s"%(self.meta_type,self.id))
  message = ''
  obj_attrs = self.getObjAttrs()
  # Attributes.
  for key in obj_attrs.keys():
    obj_attr = self.getObjAttr(key)
    datatype = obj_attr['datatype_key']
    if datatype in _globals.DT_BLOBS:
      for lang in self.getLanguages():
        try:
          req = {'lang':lang,'preview':'preview'}
          obj_vers = self.getObjVersion(req)
          blob = self._getObjAttrValue(obj_attr,obj_vers,lang)
          if blob is not None: 
            filepath = self.absolute_url()[len(self.getDocumentElement().absolute_url()):]
            filename = blob.getFilename()
            filename = getLangFilename(self,filename,lang)
            filename = '%s/%s/%s'%(folder,filepath,filename)
            filename = _fileutil.getOSPath(filename)
            _fileutil.exportObj(blob,filename)
        except:
          _globals.writeLog("*****[%s.recurse_downloadRessources]: Exception in %s.%s_%s!*****"%(self.meta_type,self.id,key,lang))
          _globals.writeLog("-  Type: %s"%str(sys.exc_type))
          _globals.writeLog("-  Value: %s"%str(sys.exc_value))
          _globals.writeLog("-  Traceback: %s"%str(sys.exc_traceback))
          pass
  # Process children.
  for child in self.getChildNodes():
    message = message + recurse_downloadRessources(child,folder)
  # Return message.
  return message


#----------------------------------------------------------------------------------------------
# _blobfields.recurse_uploadRessources:
#
# Upload from file-system to ZODB during Import.
#----------------------------------------------------------------------------------------------
def recurse_uploadRessources(self, folder='.'):
  #++ _globals.writeLog("[%s.recurse_uploadRessources]: %s"%(self.meta_type,self.id))
  message = ''
  obj_attrs = self.getObjAttrs()
  # Attributes.
  for key in obj_attrs.keys():
    obj_attr = self.getObjAttr(key)
    datatype = obj_attr['datatype_key']
    if datatype in _globals.DT_BLOBS:
      for lang in self.getLanguages():
        try:
          req = {'lang':lang,'preview':'preview'}
          obj_vers = self.getObjVersion(req)
          blob = self._getObjAttrValue(obj_attr,obj_vers,lang)
          if blob is not None:
            filename = "%s/%s"%(folder,blob.filename)
            filename = _fileutil.getOSPath(filename)
            # Backup properties (otherwise manage_upload sets it).
            bk = {}
            for __xml_attr__ in blob.__xml_attrs__:
              if hasattr(blob,__xml_attr__):
                bk[__xml_attr__] = getattr(blob,__xml_attr__)
            # Read file to ZODB.
            f = open(filename,'rb')
            uploadBlobField(self,blob,f.read(),blob.filename)
            f.close()
            # Restore properties.
            for __xml_attr__ in blob.__xml_attrs__:
              if bk.get(__xml_attr__,'') not in ['','text/x-unknown-content-type']:
                setattr(blob,__xml_attr__,bk[__xml_attr__])
            self.setObjProperty(key,blob,lang)
            self.onChangeObj(req,forced=1)
        except:
          _globals.writeLog("*****[%s.recurse_uploadRessources]: Exception in %s.%s_%s!*****"%(self.meta_type,self.id,key,lang))
          _globals.writeLog("-  Type: %s"%str(sys.exc_type))
          _globals.writeLog("-  Value: %s"%str(sys.exc_value))
          _globals.writeLog("-  Traceback: %s"%str(sys.exc_traceback))
          pass
  # Process children.
  for child in self.getChildNodes():
    message = message + recurse_uploadRessources(child,folder)
  # Return message.
  return message


#--------------------------------------------------------------------------------------------------
# createBlobField
#--------------------------------------------------------------------------------------------------
def createBlobField(self, objtype, file=''):
  blob = None
  i = MyImage(id='',title='',file='')
  f = MyFile(id='',title='',file='')
  if objtype == _globals.DT_IMAGE or objtype == str(i):
    blob = i
  elif objtype == _globals.DT_FILE or objtype == str(f):
    blob = f
  if blob is not None:
    blob.mediadbfile = None
    if type(file) is type({}):
      uploadBlobField(self,blob,StringIO(file.get('data','')),file.get('filename',''))
      if file.has_key('content_type'): blob.content_type = file.get('content_type')
    elif file:
      uploadBlobField(self,blob,file,file.filename)
  return blob


#--------------------------------------------------------------------------------------------------
# uploadBlobField
#--------------------------------------------------------------------------------------------------
def uploadBlobField(self, blob, data, filename):
  if data:
    filename = _fileutil.extractFileName(filename)
    blob.filename = filename
    blob.uploadData(data)
    if self is not None:
      mediadb = self.getMediaDb()
      if mediadb is not None:
        blob.mediadbfile = mediadb.storeFile(blob)
        blob.data = ''


#--------------------------------------------------------------------------------------------------
# getLangFilename
#--------------------------------------------------------------------------------------------------
def getLangFilename(self, filename, lang):
  i = filename.rfind('.')
  name = filename[:i]
  ext = filename[i+1:]
  if len(self.getLanguages()) > 1 and lang is not None:
    suffix = '_' + lang
    if len(name) > len(suffix) and name[-len(suffix):] != suffix:
      name = name + suffix
  return name + '.' + ext


#--------------------------------------------------------------------------------------------------
# synchronizeBlobFields
#--------------------------------------------------------------------------------------------------
def synchronizeBlobFields(self):
  #++ if _globals.debug: print "[%s.synchronizeBlobFields]:"%(self.meta_type)

  # Delete DTML Methods.
  title = 'blob_html'
  ids = map(lambda ob: ob.id(),filter(lambda ob: ob.title==title,self.objectValues(['DTML Method'])))
  if len(ids) > 0:
    try:
      #++ if _globals.debug: print " * Delete DTML Methods %s..."%str(ids)
      self.manage_delObjects(ids)
    except:
      #++ if _globals.debug: print " * Exception when deleting DTML Methods %s!"%str(ids)
      pass

  # Create DTML Methods.
  for key in self.getObjAttrs().keys():
    obj_attr = self.getObjAttr(key)
    datatype = obj_attr['datatype_key']
    if datatype in _globals.DT_BLOBS:
      for lang in self.getLanguages():
        for obj_vers in self.getObjVersions():
          v = self._getObjAttrValue(obj_attr,obj_vers,lang)
          if v is not None:
          
            # DTML call returns blob_html.
            raw = ''
  	    raw = raw + '<dtml-call \"REQUEST.set(\'lang\',\'%s\')\">\n'%lang
  	    raw = raw + '<dtml-return "blob_html(\'%s\',REQUEST,RESPONSE)">\n'%key
  	    
            # Add DTML Method with language suffix.
            filename = v.filename
            for ch in ['%','/','\\',' ','!','#','"']:
              filename = filename.replace(ch,'')
            try:
              #++ if _globals.debug: print " * Add DTML Method <%s>."%filename
              self.manage_addDTMLMethod(filename,title,raw)
            except:
              #++ if _globals.debug: print " * Exception when adding DTML Method <%s>!"%filename
              pass
              
            # Add DTML Method with language suffix.
            filename = getLangFilename(self,filename,lang)
            try:
              #++ if _globals.debug: print " * Add DTML Method with language suffix <%s>."%filename
              self.manage_addDTMLMethod(filename,title,raw)
            except:
              #++ if _globals.debug: print " * Exception when adding DTML Method with language suffix <%s>!"%filename
              pass

###################################################################################################
###################################################################################################

#--------------------------------------------------------------------------------------------------
# thumbnailImageFields
#--------------------------------------------------------------------------------------------------
def thumbnailImageFields(self, lang, manage_lang,REQUEST):
  message = ""
  if self.getConfProperty('InstalledProducts.pil',0) and \
     self.getConfProperty('InstalledProducts.pil.thumbnail',0):
    obj_attrs = self.getObjAttrs()
    for key in obj_attrs.keys():
      obj_attr = self.getObjAttr(key)
      datatype = obj_attr['datatype_key']
      if datatype == _globals.DT_IMAGE:
        message = message + thumbnailImage(self,'%ssuperres'%key,'%shires'%key,self.getConfProperty('InstalledProducts.pil.hires.thumbnail.max',600),lang,manage_lang,REQUEST)
        message = message + thumbnailImage(self,'%shires'%key,'%s'%key,self.getConfProperty('InstalledProducts.pil.thumbnail.max',100),lang,manage_lang,REQUEST)
  return message

#--------------------------------------------------------------------------------------------------
# thumbnailImage
#--------------------------------------------------------------------------------------------------
def thumbnailImage(self, hiresKey, loresKey, maxdim, lang, manage_lang, REQUEST):
  message = ""
  if 1==1:
    if hiresKey in self.getObjAttrs().keys():
      hiresImg = self.getObjProperty(hiresKey,REQUEST)
      if hiresImg is not None and REQUEST.get('generate_preview_%s_%s'%(hiresKey,lang),0):
        if _globals.debug: print "[%s.manage_changeProperties]: Create >%s< from >%s<..."%(self.meta_type,loresKey,hiresKey)
        loresImg = _fileutil.createThumbnail(hiresImg,maxdim)
        self.setObjProperty(loresKey,loresImg,lang)
  try:
    pass
  except:
    pass
  return message


###################################################################################################
###################################################################################################

class MyBlob:

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getData: 
    # ---------------------------------------------------------------------------------------------
    getData__roles__ = None
    def getData(self):
      data = ''
      mediadbfile = getattr(self,'mediadbfile',None)
      if mediadbfile is not None:
        parent = self.aq_parent
        mediadb = parent.getMediaDb()
        if mediadb is not None:
          try:
            data = mediadb.retrieveFile(mediadbfile)
          except:
            pass
      else:
        data = str(getattr(self,'data',''))
      return data
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.blob_html: 
    # ---------------------------------------------------------------------------------------------
    blob_html__roles__ = None
    def blob_html(self, REQUEST, RESPONSE):
      filename = self.getFilename()
      content_type = getattr(self,'content_type','application/octet-stream')
      RESPONSE.setHeader('Content-Type',content_type)
      RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
      return self.getData()
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.src: 
    # 
    #   DEPRECATED!
    # ---------------------------------------------------------------------------------------------
    src__roles__ = None
    def src(self, parent, key, REQUEST):
      qs = ''
      ids = parent.objectIds(['DTML Method'])
      filename = self.getFilename()
      filename = getLangFilename(parent,filename,self.lang)
      if not filename in ids:
        filename = 'blob_html'
        qs = _globals.qs_append(qs,'key',key)
        qs = _globals.qs_append(qs,'lang',REQUEST['lang'])
      if REQUEST.get('ZMS_VERSION',None) is not None:
        qs = _globals.qs_append(qs,'ZMS_VERSION',REQUEST['ZMS_VERSION'])
      elif _globals.isPreviewRequest(REQUEST):
        qs = _globals.qs_append(qs,'preview','preview')
      return '%s/%s%s'%(parent.absolute_url(),filename,qs)

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getHref:
    # ---------------------------------------------------------------------------------------------
    getHref__roles__ = None
    def getHref(self, REQUEST):
      parent = self.aq_parent
      key = self.key
      return self.src(parent,key,REQUEST)

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getFilename: 
    #
    #	Returns filename.
    # ---------------------------------------------------------------------------------------------
    getFilename__roles__ = None
    def getFilename(self):
      filename = _fileutil.extractFileName(self.filename) 
      filename = filename.replace('%','')
      filename = filename.replace(' ','')
      return filename
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.get_size: 
    #
    #	Returns file-size.
    # ---------------------------------------------------------------------------------------------
    get_size__roles__ = None
    def get_size(self):
      size = 0
      data = self.getData()
      if data is not None:
        size = len(data)
      return size
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getDataSizeStr: 
    #
    #	Returns display string of file-size (KB).
    # ---------------------------------------------------------------------------------------------
    getDataSizeStr__roles__ = None
    def getDataSizeStr(self):
      return _fileutil.getDataSizeStr(self.get_size())

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getContentType:
    #
    #	MIME-type.
    # ---------------------------------------------------------------------------------------------
    getContentType__roles__ = None
    def getContentType(self):
      return self.content_type

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getMimeTypeIconSrc:
    #
    #	Image source for MIME-type.
    # ---------------------------------------------------------------------------------------------
    getMimeTypeIconSrc__roles__ = None
    def getMimeTypeIconSrc(self):
      content_type = self.getContentType()
      if not _mimetypes.dctMimeType.has_key(content_type):
        content_type = 'content/unknown'
      return '%s%s'%(self.aq_parent.MISC_ZMS,_mimetypes.dctMimeType[content_type])

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.xmlGetTagName: 
    #
    #	Returns <XML> Tag-Name.
    # ---------------------------------------------------------------------------------------------
    def xmlGetTagName(self):
      return 'data'


###################################################################################################
###################################################################################################

class MyImage(MyBlob,Image):

    __obj_attrs__ = ['size','width','height','content_type','data','filename','mediadbfile']
    __xml_attrs__ = ['width','height']
    
    def __str__(self):
      return '{{MyImage}}'

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.uploadData:
    # ---------------------------------------------------------------------------------------------
    def uploadData(self, data):
      ob = Image(id='',title='',file='')
      ob.manage_upload(data)
      for key in ['size','width','height','content_type','data']:
        if hasattr(ob,key):
          setattr(self,key,getattr(ob,key))
    
    # ---------------------------------------------------------------------------------------------
    # 	MyImage._getCopy:
    # ---------------------------------------------------------------------------------------------
    def _getCopy(self):
      ob = self
      clone = MyImage(id='',title='',file='')
      attrs = self.__obj_attrs__
      for attr in attrs:
        if hasattr(ob,attr):
          setattr(clone,attr,getattr(ob,attr))
      return clone

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.toXml:
    # ---------------------------------------------------------------------------------------------
    def toXml(self, sender=None):
      data = ''
      objtype = ''
      filename = _fileutil.getOSPath(_fileutil.extractFileName(getattr(self,'filename','')))
      if sender is None:
        data = bin2hex(self.getData())
        objtype = ' type="image"'
      else:
        filepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url())+1:]
        filename = self.getFilename()
        filename = getLangFilename(sender,filename,self.lang)
        filename = '%s/%s'%(filepath,filename)
      xml = '\n<%s'%self.xmlGetTagName()
      xml = xml + ' width="%s"'%str(getattr(self,'width',''))
      xml = xml + ' height="%s"'%str(getattr(self,'height',''))
      xml = xml + ' content_type="%s"'%str(getattr(self,'content_type',''))
      xml = xml + ' filename="%s"'%filename
      xml = xml + objtype + '>' + data
      xml = xml + '</%s>'%self.xmlGetTagName()
      return xml

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.getWidth:
    # ---------------------------------------------------------------------------------------------
    getWidth__roles__ = None
    def getWidth(self):
      return self.width

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.getHeight:
    # ---------------------------------------------------------------------------------------------
    getHeight__roles__ = None
    def getHeight(self):
      return self.height


###################################################################################################
###################################################################################################

class MyFile(MyBlob,File):

    __obj_attrs__ = ['size','content_type','data','filename','mediadbfile']
    __xml_attrs__ = []

    def __str__(self):
      return '{{MyFile}}'

    # ---------------------------------------------------------------------------------------------
    # 	MyFile.uploadData:
    # ---------------------------------------------------------------------------------------------
    def uploadData(self, data):
      ob = File(id='',title='',file=data)
      for key in ['content_type','data']:
        if hasattr(ob,key):
          setattr(self,key,getattr(ob,key))
    
    # ---------------------------------------------------------------------------------------------
    # 	MyFile._getCopy:
    # ---------------------------------------------------------------------------------------------
    def _getCopy(self):
      ob = self
      clone = MyFile(id='',title='',file='')
      attrs = self.__obj_attrs__
      for attr in attrs:
        if hasattr(ob,attr):
          setattr(clone,attr,getattr(ob,attr))
      return clone

    # ---------------------------------------------------------------------------------------------
    # 	MyFile.toXml:
    # ---------------------------------------------------------------------------------------------
    def toXml(self, sender=None):
      data = ''
      objtype = ''
      filename = _fileutil.getOSPath(_fileutil.extractFileName(getattr(self,'filename','')))
      if sender is None:
        data = bin2hex(self.getData())
        objtype = ' type="file"'
      else:
        filepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url())+1:]
        filename = self.getFilename()
        filename = getLangFilename(sender,filename,self.lang)
        filename = '%s/%s'%(filepath,filename)
      xml = '\n<%s'%self.xmlGetTagName()
      xml = xml + ' content_type="%s"'%str(getattr(self,'content_type',''))
      xml = xml + ' filename="%s"'%filename
      xml = xml + objtype + '>' + data
      xml = xml + '</%s>'%self.xmlGetTagName()
      return xml

###################################################################################################
