###################################################################################################
# _zcatalogmanager.py
#
# Implementation of classes ZCatalogItem and ZCatalogManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Products.ZCatalog import ZCatalog, Vocabulary
from App.version_txt import version_txt
import time
import urllib
# Product Imports.
import _globals

# -------------------------------------------------------------------------------------------------
# search_string:
#
# Search string of given value.
# -------------------------------------------------------------------------------------------------
def search_string(v):
  s = ''
  if v is not None:
    if type(v) is type('') and len(v) > 0:
      s = s + v + ' '
    elif type(v) is type([]):
      for i in v:
        s = s + search_string(i)
    elif type(v) is type({}):
      for k in v.keys():
        i = v[k]
        s = s + search_string(i)
  return s

# -------------------------------------------------------------------------------------------------
# search_quote:
#
# Remove HTML-Tags from given string.
# -------------------------------------------------------------------------------------------------
def search_quote(s, maxlen=255, tag='&middot;'):
  # remove all tags.
  stop = 0
  while not stop:
    iStartTag = s.find('<')
    iEndTag = s.find('>',iStartTag)
    if iStartTag >= 0 and iEndTag > iStartTag:
      s = s[:iStartTag] + tag + s[iEndTag+1:]
    else:
      stop = 1
  # limit characters.
  if len(s) > maxlen:
    blank = s.find(' ',maxlen)
    s = s[:blank] + "&middot;&middot;&middot;"
  # search quote.
  while s.find('>') >= 0:
    i = s.find('>')
    s = s[:i] + '&gt;' + s[i+1:]
  while s.find('<') >= 0:
    i = s.find('<')
    s = s[:i] + '&lt;' + s[i+1:]
  # return quoted search string.
  return s

# -------------------------------------------------------------------------------------------------
# _synchronizeCatalogText
# -------------------------------------------------------------------------------------------------
def _synchronizeCatalogText(self, REQUEST):
  if self.meta_type == 'ZMSTrashcan': return
  el_meta_types = ['ZMS','ZMSRubrik','ZMSDocument','ZMSCustom','ZMSSysFolder']
  for s_lang in self.getLanguages():
    req = {'lang':s_lang,'manage_lang':s_lang}
    zcat_text = ""
    if self.isPage():
      zcat_text = self.catalogText(req)
    setattr(self,'zcat_text_%s'%s_lang,zcat_text)
  if self.isPage():
    for child in self.getChildNodes(REQUEST,el_meta_types):
      _synchronizeCatalogText(child,REQUEST)
        
        
###################################################################################################
###################################################################################################
###
###   class ZCatalogItem
###
###################################################################################################
###################################################################################################
class ZCatalogItem: 

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogItem.search_quote:
    #
    #	Remove HTML-Tags.
    # ---------------------------------------------------------------------------------------------
    def search_quote(self, s, maxlen=255, tag="&middot;"):
      return search_quote(s,maxlen,tag)
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getCatalogNavUrl:
    #
    #	Returns catalog-navigation url.
    # ---------------------------------------------------------------------------------------------
    def getCatalogNavUrl(self,REQUEST):
      s = REQUEST['URL']
      s = '%s?op=%s'%(s,urllib.quote(REQUEST.get('op','search')))
      s = '%s&raw=%s'%(s,urllib.quote(REQUEST.get('raw','')))
      s = '%s&option=%s'%(s,urllib.quote(REQUEST.get('option','')))
      s = '%s&search_order_by=%s'%(s,urllib.quote(str(REQUEST.get('search_order_by',0))))
      return s

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogItem.catalogText:
    #
    #	Catalog text.
    # ---------------------------------------------------------------------------------------------
    def catalogText(self, REQUEST):
      if _globals.debug: print '[%s.catalogText]: %s'%(self.meta_type,self.absolute_url())
      s = ''
      if self.isVisible(REQUEST):
        for key in self.getObjAttrs().keys():
          obj_attr = self.getObjAttr(key)
          if obj_attr['xml']:
            datatype = obj_attr['datatype_key']
            if datatype in _globals.DT_STRINGS or \
               datatype == _globals.DT_DICTIONARY or \
               datatype == _globals.DT_LIST:
              value = self.getObjAttrValue(obj_attr,REQUEST)
              s = s + search_string(value)
        for ob in self.getChildNodes(REQUEST,self.PAGEELEMENTS):
          s = s + ob.catalogText(REQUEST)
      return s


###################################################################################################
###################################################################################################
###
###   class ZCatalogManager
###
###################################################################################################
###################################################################################################
class ZCatalogManager: 

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.recreateCatalog:
    #
    #	(Re)-creates catalog-object.
    # ---------------------------------------------------------------------------------------------
    def recreateCatalog(self):
      
      ##### Remove ZCatalog ####
      if _globals.debug: print "[%s.recreateCatalog]: Remove ZCatalog..."%(self.meta_type)
      ids = self.objectIds(['ZCatalog'])
      if len(ids) > 0:
        self.manage_delObjects(ids,None)
      
      ##### Add ZCatalog ####
      if _globals.debug: print "[%s.recreateCatalog]: Add ZCatalog..."%(self.meta_type)
      cat_id = 'catalog'
      cat_title = 'Catalog'
      vocab_id = 'create_default_catalog_'
      zcatalog = ZCatalog.ZCatalog(cat_id, cat_title, vocab_id, self)
      self._setObject(zcatalog.id, zcatalog)
      
      ##### Remove Default-Vocabulary ####
      if _globals.debug: print "[%s.recreateCatalog]: Remove Default-Vocabulary..."%(self.meta_type)
      ids = zcatalog.objectIds(['Vocabulary'])
      if len(ids) > 0:
        zcatalog.manage_delObjects(ids,None)
      
      ##### Add ISO 8859-1 Vocabulary ####
      # see ZCatalogs with Umlauts
      # http://www.zope.org/Members/strobl/HowTos/Iszcatalog
      if _globals.debug: print "[%s.recreateCatalog]: Add ISO 8859-1 Vocabulary..."%(self.meta_type)
      id = "Vocabulary"
      title = id
      globbing = 1
      splitter = "ISO_8859_1_Splitter"
      oVocabulary = Vocabulary.Vocabulary(id,title,globbing,splitter)
      zcatalog._setObject(oVocabulary.id,oVocabulary)
      return zcatalog

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.getCatalog:
    #
    #	Returns catalog-object (None if there is no catalog).
    # ---------------------------------------------------------------------------------------------
    def getCatalog(self):
      obs = self.getDocumentElement().objectValues(['ZCatalog'])
      if len(obs) == 0:
        return self.recreateCatalog()
      return obs[0]

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.synchronizeSearch:
    # ---------------------------------------------------------------------------------------------
    def synchronizeSearch(self, REQUEST):
      self.setCatalogUpdateFlag()

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.__update_catalog__:
    #
    #	Update (refresh) catalog.
    #	IN:   forced_update	0= not forced (update once a day), 
    #				1= forced (call from management-interface).
    # ---------------------------------------------------------------------------------------------
    def __update_catalog__(self, forced_update, REQUEST):
        
        if forced_update:
          self.recreateCatalog()
          
        do_update = 1
        do_update = do_update and (self.getCatalogUpdateFlag() > 0)
        do_update = do_update and (self.getCatalogPeriod() == '0' or self.getCatalogUpdate())
        do_update = do_update or forced_update
        
        # do update if desired
        if do_update:
          if _globals.debug: print '[%s.__update_catalog__]: do update!'%self.meta_type
            
          _synchronizeCatalogText(self,REQUEST)
            
          zcatalog = self.getCatalog()
          oRoot = self.getHome()
             
          # remove items from catalog
          zcatalog.manage_catalogClear()
                
          # delete indices from catalog
          zcatalog.manage_delColumn(zcatalog.schema())
          zcatalog.manage_delIndex(zcatalog.indexes())
                
          # create indices on catalog
          zcatalog.manage_addColumn('getSelf')
          for s_lang in self.getLanguages():
  	    key = 'zcat_text_%s'%s_lang
	    zcatalog.manage_addColumn(key)
  	    zcatalog.manage_addIndex(key,'TextIndex')
  		  
          # find items to catalog
          path = oRoot.absolute_url()[len(str(REQUEST['BASE0']))+1:]
          results = zcatalog.ZopeFindAndApply(oRoot,
                     obj_metatypes=['ZMS','ZMSRubrik','ZMSDocument','ZMSCustom','ZMSSysFolder'],
                     obj_ids=None,
                     obj_searchterm=None,
                     obj_expr=None,
                     obj_mtime=None,
                     obj_mspec=None,
                     obj_permission=None,
                     obj_roles=None,
                     search_sub=1,
                     REQUEST=REQUEST,
                     apply_func=zcatalog.catalog_object,
                     apply_path=path)
                           
          # update time-stamp of last-catalog-update
          self.clearCatalogUpdateFlag()


    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.getCatalogQueryString:
    # ---------------------------------------------------------------------------------------------
    def getCatalogQueryString(self, raw, option='AND'):
      qs = ''
      q = 0
      c = 0
      for raw_item in raw.split(' '):
        raw_item = raw_item.strip() 
        if len(raw_item)>0:
          if q==0 and c>0:
            qs='%s %s '%(qs,option)
          if raw_item[0]=='*':
            q=1
          if q==1:
            qs='%s %s '%(qs,raw_item)
          else:
            qs='%s *%s* '%(qs,raw_item)
          if raw_item[-1]=='*':
            q=0
        c=c+1
      return qs


    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.submitCatalogQuery:
    #
    #	Submits query to catalog.
    # ---------------------------------------------------------------------------------------------
    def submitCatalogQuery(self, search_query, search_order_by, search_meta_types, REQUEST):
    
      # Update ZCatalog (0= not forced, update only if necessary)
      self.__update_catalog__(0, REQUEST)

      # Numbers are not cataloged.
      lnum = []
      lq = search_query.replace('-','* AND *').split(' ')
      qs = ''
      for i in range(0,len(lq)):
        s = lq[i]
        try: 
          v = int(s[1:-1])
          lnum.append(v)
        except:
          if len(s) > 3: 
            if len(qs) > 0:
              qs = qs + lq[i-1] + ' '
            qs = qs + s + ' '
        
      # Search ZCatalog.
      zcatalog = self.getCatalog()
      meta_type = ['ZMS','ZMSRubrik','ZMSDocument','ZMSCustom','ZMSSysFolder']    
      rtn = zcatalog({'meta_type':meta_type,'zcat_text_%s'%REQUEST['lang']:qs})

      # Sort order.
      if int(search_order_by)==1:
        sOrderBy = 'score'
      else:
        sOrderBy = 'time'
        
      # Sort result-set.
      obs = []
      for oItem in rtn:
        sDataRecordId = oItem.data_record_id_
        sPath = REQUEST['BASE0'] + "/" + zcatalog.getpath(sDataRecordId)
        ob = self
        for id in zcatalog.getpath(sDataRecordId).split('/'):
          if ob is not None:
            ob = getattr(ob,id,None)
        if ob is None:
          ob = oItem.getSelf
        description = getattr(ob,'zcat_text_%s'%REQUEST['lang'],'')
        if _globals.debug: print "[%s]"%(ob.meta_type)
        append = 1
        append = append and ob.isPage()
        # Handle Meta-Types.
        if len(search_meta_types) > 0:
          if meta_type == 'ZMSCustom':
            append = append and ob.meta_id in search_meta_types
          else:
            append = append and ob.meta_type in search_meta_types
        # Handle Numbers.
        for num in lnum:
          append = append and description.find(str(num)) >= 0
        if append:
          sum_time = ob.getObjProperty('attr_dc_date',REQUEST)
          if sum_time is None or len(str(sum_time))==0:
            sum_time = ob.getObjProperty('change_dt',REQUEST)
          sum_url = sPath + "/index_%s.html"%REQUEST['lang']
          if REQUEST.get('preview','') == 'preview':
            sum_url = self.url_append_params(sum_url,{'preview':'preview'})
          if ob.meta_type == 'ZMSCustom':
            sum_url = self.url_append_params(sum_url,{'ZMS_PARAM0':'search','ZMS_PARAM1':qs})
          summary = {}
          summary['ob'] = ob
          summary['title'] = ob.getTitle(REQUEST)
          summary['description'] = ob.getDCDescription(REQUEST)
          summary['score'] = int(oItem.data_record_score_)
          summary['time'] = sum_time
          summary['url'] = sum_url
          obs.append((summary[sOrderBy],summary))

      # Sort objects.
      obs.sort()
      obs.reverse()
        
      # Return objects in correct sort-order.
      return map(lambda ob: ob[1],obs)


    ###############################################################################################
    # Catalog Update Period
    ###############################################################################################

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.getCatalogPeriod: 
    # ------------------------------------------------------------------------------------------
    def getCatalogPeriod(self):
      return self.getConfProperty('ZMS.catalog.period','0')

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.setCatalogPeriod: 
    # ------------------------------------------------------------------------------------------
    def setCatalogPeriod(self, v='1d'):
      self.setConfProperty('ZMS.catalog.period',v)


    ###############################################################################################
    # Catalog Update Flag
    ###############################################################################################

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.clearCatalogUpdateFlag: 
    # ------------------------------------------------------------------------------------------
    def clearCatalogUpdateFlag(self):
      setattr(self,'update_catalog_flag',0)
      setattr(self,'update_catalog_time',time.time())
    
    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.setCatalogUpdateFlag: 
    # ------------------------------------------------------------------------------------------
    def setCatalogUpdateFlag(self):
      setattr(self,'update_catalog_flag',self.getCatalogUpdateFlag()+1)

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.getCatalogUpdateFlag: 
    # ------------------------------------------------------------------------------------------
    def getCatalogUpdateFlag(self):
      return getattr(self,'update_catalog_flag',0)

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.getCatalogUpdateTime: 
    # ------------------------------------------------------------------------------------------
    def getCatalogUpdateTime(self):
      return getattr(self,'update_catalog_time',None)

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.getCatalogUpdate: 
    # ------------------------------------------------------------------------------------------
    def getCatalogUpdate(self):
      t_now = time.time()
      t_last_update = self.getCatalogUpdateTime()
      return t_last_update is None or _globals.compareDate(t_now,t_last_update) == 1


    ###############################################################################################
    # ZCatalogManager.manage_updateCatalog:
    #
    # Update (refresh) catalog manually.
    ###############################################################################################
    def manage_updateCatalog(self, btn, lang, manage_lang, REQUEST):
      """ ZMSObject.manage_updateCatalog """
      
      # Change.
      # -------
      if btn == 'Change': 
        self.setCatalogPeriod(REQUEST.get('period','0'))
        
      # Refresh.
      # -------
      elif btn == 'Refresh':
        self.__update_catalog__(1,REQUEST) # Update ZCatalog (1=forced)
          
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return REQUEST.RESPONSE.redirect('manage_customizeForm?lang=%s&manage_lang=%s&manage_tabs_message=%s#_search'%(lang,manage_lang,message))
      
###################################################################################################
